@@ -4,7 +4,7 @@ from django import http
 from django.contrib import messages
 from django.core.exceptions import ImproperlyConfigured
 from django.urls import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from oscar.core import prices
 from oscar.core.loading import get_class, get_model
@@ -12,6 +12,7 @@ from oscar.core.loading import get_class, get_model
 from . import exceptions
 
 Repository = get_class('shipping.repository', 'Repository')
+SurchargeApplicator = get_class("checkout.applicator", "SurchargeApplicator")
 OrderTotalCalculator = get_class(
     'checkout.calculators', 'OrderTotalCalculator')
 CheckoutSessionData = get_class(
@@ -66,7 +67,7 @@ class CheckoutSessionMixin(object):
         except exceptions.PassedSkipCondition as e:
             return http.HttpResponseRedirect(e.url)
 
-        return super(CheckoutSessionMixin, self).dispatch(
+        return super().dispatch(
             request, *args, **kwargs)
 
     def check_pre_conditions(self, request):
@@ -228,6 +229,7 @@ class CheckoutSessionMixin(object):
         shipping_address = self.get_shipping_address(request.basket)
         shipping_method = self.get_shipping_method(
             request.basket, shipping_address)
+        surcharges = SurchargeApplicator(request).get_applicable_surcharges(basket=request.basket)
         if shipping_method:
             shipping_charge = shipping_method.calculate(request.basket)
         else:
@@ -238,7 +240,7 @@ class CheckoutSessionMixin(object):
                 currency=request.basket.currency, excl_tax=D('0.00'),
                 tax=D('0.00')
             )
-        total = self.get_order_totals(request.basket, shipping_charge)
+        total = self.get_order_totals(request.basket, shipping_charge, surcharges)
         if total.excl_tax == D('0.00'):
             raise exceptions.PassedSkipCondition(
                 url=reverse('checkout:preview')
@@ -249,7 +251,7 @@ class CheckoutSessionMixin(object):
     def get_context_data(self, **kwargs):
         # Use the proposed submission as template context data.  Flatten the
         # order kwargs so they are easily available too.
-        ctx = super(CheckoutSessionMixin, self).get_context_data()
+        ctx = super().get_context_data()
         ctx.update(self.build_submission(**kwargs))
         ctx.update(kwargs)
         ctx.update(ctx['order_kwargs'])
@@ -270,22 +272,27 @@ class CheckoutSessionMixin(object):
         shipping_method = self.get_shipping_method(
             basket, shipping_address)
         billing_address = self.get_billing_address(shipping_address)
-        if not shipping_method:
-            total = shipping_charge = None
-        else:
-            shipping_charge = shipping_method.calculate(basket)
-            total = self.get_order_totals(
-                basket, shipping_charge=shipping_charge, **kwargs)
         submission = {
             'user': self.request.user,
             'basket': basket,
             'shipping_address': shipping_address,
             'shipping_method': shipping_method,
-            'shipping_charge': shipping_charge,
             'billing_address': billing_address,
-            'order_total': total,
             'order_kwargs': {},
-            'payment_kwargs': {}}
+            'payment_kwargs': {}
+        }
+
+        surcharges = SurchargeApplicator(self.request, submission).get_applicable_surcharges(self.request.basket)
+        if not shipping_method:
+            total = shipping_charge = None
+        else:
+            shipping_charge = shipping_method.calculate(basket)
+            total = self.get_order_totals(
+                basket, shipping_charge=shipping_charge, surcharges=surcharges, **kwargs)
+
+        submission["shipping_charge"] = shipping_charge
+        submission["order_total"] = total
+        submission['surcharges'] = surcharges
 
         # If there is a billing address, add it to the payment kwargs as calls
         # to payment gateways generally require the billing address. Note, that
@@ -301,8 +308,8 @@ class CheckoutSessionMixin(object):
         # Set guest email after overrides as we need to update the order_kwargs
         # entry.
         user = submission['user']
-        if (not user.is_authenticated and
-                'guest_email' not in submission['order_kwargs']):
+        if (not user.is_authenticated
+                and 'guest_email' not in submission['order_kwargs']):
             email = self.checkout_session.get_guest_email()
             submission['order_kwargs']['guest_email'] = email
         return submission
@@ -408,9 +415,9 @@ class CheckoutSessionMixin(object):
                 user_address.populate_alternative_model(billing_address)
                 return billing_address
 
-    def get_order_totals(self, basket, shipping_charge, **kwargs):
+    def get_order_totals(self, basket, shipping_charge, surcharges=None, **kwargs):
         """
         Returns the total for the order with and without tax
         """
         return OrderTotalCalculator(self.request).calculate(
-            basket, shipping_charge, **kwargs)
+            basket, shipping_charge, surcharges, **kwargs)
