@@ -10,7 +10,7 @@ from django.db.models import Count, Q, Sum, fields
 from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.shortcuts import get_object_or_404, redirect
 from django.urls import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.views.generic import DetailView, FormView, ListView, UpdateView
 
 from oscar.apps.order import exceptions as order_exceptions
@@ -50,7 +50,7 @@ def queryset_orders_for_user(user):
         'billing_address', 'billing_address__country',
         'shipping_address', 'shipping_address__country',
         'user'
-    ).prefetch_related('lines')
+    ).prefetch_related('lines', 'status_changes')
     if user.is_staff:
         return queryset
     else:
@@ -70,7 +70,7 @@ class OrderStatsView(FormView):
     Dashboard view for order statistics.
     Supports the permission-based dashboard.
     """
-    template_name = 'dashboard/orders/statistics.html'
+    template_name = 'oscar/dashboard/orders/statistics.html'
     form_class = OrderStatsForm
 
     def get(self, request, *args, **kwargs):
@@ -82,12 +82,12 @@ class OrderStatsView(FormView):
         return self.render_to_response(ctx)
 
     def get_form_kwargs(self):
-        kwargs = super(OrderStatsView, self).get_form_kwargs()
+        kwargs = super().get_form_kwargs()
         kwargs['data'] = self.request.GET
         return kwargs
 
     def get_context_data(self, **kwargs):
-        ctx = super(OrderStatsView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         filters = kwargs.get('filters', {})
         ctx.update(self.get_stats(filters))
         ctx['title'] = kwargs['form'].get_filter_description()
@@ -113,7 +113,7 @@ class OrderListView(BulkEditMixin, ListView):
     """
     model = Order
     context_object_name = 'orders'
-    template_name = 'dashboard/orders/order_list.html'
+    template_name = 'oscar/dashboard/orders/order_list.html'
     form_class = OrderSearchForm
     paginate_by = settings.OSCAR_DASHBOARD_ITEMS_PER_PAGE
     actions = ('download_selected_orders', 'change_order_statuses')
@@ -122,7 +122,7 @@ class OrderListView(BulkEditMixin, ListView):
         # base_queryset is equal to all orders the user is allowed to access
         self.base_queryset = queryset_orders_for_user(
             request.user).order_by('-date_placed')
-        return super(OrderListView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
     def get(self, request, *args, **kwargs):
         if 'order_number' in request.GET and request.GET.get(
@@ -136,7 +136,7 @@ class OrderListView(BulkEditMixin, ListView):
             else:
                 return redirect(
                     'dashboard:order-detail', number=order.number)
-        return super(OrderListView, self).get(request, *args, **kwargs)
+        return super().get(request, *args, **kwargs)
 
     def get_queryset(self):  # noqa (too complex (19))
         """
@@ -333,7 +333,7 @@ class OrderListView(BulkEditMixin, ListView):
         return descriptions
 
     def get_context_data(self, **kwargs):
-        ctx = super(OrderListView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         ctx['form'] = self.form
         ctx['order_statuses'] = Order.all_statuses()
         ctx['search_filters'] = self.get_search_filter_descriptions()
@@ -350,7 +350,7 @@ class OrderListView(BulkEditMixin, ListView):
             return self.download_selected_orders(
                 self.request,
                 context['object_list'])
-        return super(OrderListView, self).render_to_response(
+        return super().render_to_response(
             context, **response_kwargs)
 
     def get_download_filename(self, request):
@@ -435,7 +435,7 @@ class OrderDetailView(DetailView):
     """
     model = Order
     context_object_name = 'order'
-    template_name = 'dashboard/orders/order_detail.html'
+    template_name = 'oscar/dashboard/orders/order_detail.html'
 
     # These strings are method names that are allowed to be called from a
     # submitted form.
@@ -523,7 +523,7 @@ class OrderDetailView(DetailView):
         return HttpResponseRedirect(url)
 
     def get_context_data(self, **kwargs):
-        ctx = super(OrderDetailView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         ctx['active_tab'] = kwargs.get('active_tab', 'lines')
 
         # Forms
@@ -607,7 +607,7 @@ class OrderDetailView(DetailView):
             messages.error(
                 request, _("Unable to change order status due to "
                            "payment error: %s") % e)
-        except order_exceptions.InvalidOrderStatus as e:
+        except order_exceptions.InvalidOrderStatus:
             # The form should validate against this, so we should only end up
             # here during race conditions.
             messages.error(
@@ -737,7 +737,7 @@ class LineDetailView(DetailView):
     """
     model = Line
     context_object_name = 'line'
-    template_name = 'dashboard/orders/line_detail.html'
+    template_name = 'oscar/dashboard/orders/line_detail.html'
 
     def get_object(self, queryset=None):
         order = get_order_for_user_or_404(self.request.user,
@@ -748,7 +748,7 @@ class LineDetailView(DetailView):
             raise Http404()
 
     def get_context_data(self, **kwargs):
-        ctx = super(LineDetailView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         ctx['order'] = self.object.order
         return ctx
 
@@ -794,7 +794,7 @@ class ShippingAddressUpdateView(UpdateView):
     """
     model = ShippingAddress
     context_object_name = 'address'
-    template_name = 'dashboard/orders/shippingaddress_form.html'
+    template_name = 'oscar/dashboard/orders/shippingaddress_form.html'
     form_class = ShippingAddressForm
 
     def get_object(self, queryset=None):
@@ -803,13 +803,13 @@ class ShippingAddressUpdateView(UpdateView):
         return get_object_or_404(self.model, order=order)
 
     def get_context_data(self, **kwargs):
-        ctx = super(ShippingAddressUpdateView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         ctx['order'] = self.object.order
         return ctx
 
     def form_valid(self, form):
         old_address = ShippingAddress.objects.get(id=self.object.id)
-        response = super(ShippingAddressUpdateView, self).form_valid(form)
+        response = super().form_valid(form)
         changes = get_change_summary(old_address, self.object)
         if changes:
             msg = _("Delivery address updated:\n%s") % changes
