@@ -1,3 +1,4 @@
+from django.utils.translation import gettext_lazy as _
 from django_tables2 import A, Column, LinkColumn, TemplateColumn
 
 from oscar.core.loading import get_class
@@ -7,7 +8,7 @@ DashboardTable = get_class('dashboard.tables', 'DashboardTable')
 
 class UserTable(DashboardTable):
     check = TemplateColumn(
-        template_name='dashboard/users/user_row_checkbox.html',
+        template_name='oscar/dashboard/users/user_row_checkbox.html',
         verbose_name=' ', orderable=False)
     email = LinkColumn('dashboard:user-detail', args=[A('id')],
                        accessor='email')
@@ -16,12 +17,12 @@ class UserTable(DashboardTable):
     active = Column(accessor='is_active')
     staff = Column(accessor='is_staff')
     date_registered = Column(accessor='date_joined')
-    num_orders = Column(accessor='orders.count', orderable=False)
+    num_orders = Column(accessor='orders__count', orderable=False, verbose_name=_('Number of Orders'))
     actions = TemplateColumn(
-        template_name='dashboard/users/user_row_actions.html',
+        template_name='oscar/dashboard/users/user_row_actions.html',
         verbose_name=' ')
 
     icon = "group"
 
     class Meta(DashboardTable.Meta):
-        template = 'dashboard/users/table.html'
+        template_name = 'oscar/dashboard/users/table.html'
