@@ -1,10 +1,10 @@
 from django.db.models.signals import post_save
 from django.dispatch import receiver
 
-from oscar.core.loading import get_classes
+from oscar.core.loading import get_model
 
-StockRecord, StockAlert = get_classes('partner.models', ['StockRecord',
-                                                         'StockAlert'])
+StockAlert = get_model('partner', 'StockAlert')
+StockRecord = get_model('partner', 'StockRecord')
 
 
 @receiver(post_save, sender=StockRecord)
