@@ -10,7 +10,7 @@ Line = get_model('wishlists', 'Line')
 class WishListForm(forms.ModelForm):
 
     def __init__(self, user, *args, **kwargs):
-        super(WishListForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.instance.owner = user
 
     class Meta:
@@ -21,5 +21,5 @@ class WishListForm(forms.ModelForm):
 class WishListLineForm(forms.ModelForm):
 
     def __init__(self, *args, **kwargs):
-        super(WishListLineForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fields['quantity'].widget.attrs['class'] = 'input-mini'
