@@ -2,6 +2,57 @@
 
 var oscar = (function(o, $) {
 
+    function onFileChange(evt) {
+        var reader = new FileReader();
+        var imgId = evt.target.id + "-image";
+        reader.onload = (function() {
+            return function(e) {
+                var imgDiv = $("#" + imgId);
+                imgDiv.children('img').attr('src', e.target.result);
+            };
+        })();
+        reader.readAsDataURL(evt.target.files[0]);
+
+        var $input = $(evt.target);
+        var $parentTab = $input.parents('.tab-pane').first();
+        var imageContainer = $input.parents('.sortable-handle').first();
+        imageContainer.find('.btn-reorder').removeAttr('disabled').removeClass('disabled');
+        var $extraImg = $input.parents('.upload-image').children('li').last();
+        var $totalForms = $parentTab.find("input[name$=images-TOTAL_FORMS]");
+        var $maxForms = $parentTab.find("input[name$=images-MAX_NUM_FORMS]");
+        var numExisting = parseInt($totalForms.val());
+        var numMax = parseInt($maxForms.val());
+
+        // Do not create extra image form if number of maximum allowed forms has reached.
+        if (numExisting < numMax) {
+            var $newImg = o.dashboard._extraProductImg.clone();
+            var productId = $('#images-0-product').val();
+            $newImg.insertAfter($extraImg);
+            // update attrs on cloned el
+            $newImg.find("[id^='id_images-']," +
+                "[for^='id_images-']," +
+                "[id^='upload_button_id_images-']," +
+                "img[alt='thumbnail']").each(function () {
+                var $el = $(this);
+                ["id", "name", "for", "onload", "onerror"].forEach(function (attr) {
+                    var val = $el.attr(attr);
+                    if (val) {
+                        var parts = val.split('-');
+                        parts[1] = numExisting;
+                        $el.attr(attr, parts.join('-'));
+                    }
+                });
+            });
+            $newImg.find('#id_images-' + numExisting + '-display_order').val(numExisting);
+            $newImg.find('#id_images-' + numExisting + '-product').val(productId);
+
+            var $newFile = $newImg.find('input[type="file"]');
+            $newFile.change(onFileChange);
+            numExisting += 1;
+            $totalForms.val(numExisting);
+        }
+    }
+
     o.getCsrfToken = function() {
         // Extract CSRF token from cookies
         var cookies = document.cookie.split(';');
@@ -82,6 +133,7 @@ var oscar = (function(o, $) {
             o.dashboard.initMasks(el);
             o.dashboard.initWYSIWYG(el);
             o.dashboard.initSelects(el);
+            o.dashboard.initProductImages(el);
         },
         initMasks: function(el) {
             $(el).find(':input').inputmask();
@@ -91,7 +143,46 @@ var oscar = (function(o, $) {
             var $selects = $(el).find('select').not('.no-widget-init select').not('.no-widget-init');
             $selects.filter('.form-stacked select').css('width', '95%');
             $selects.filter('.form-inline select').css('width', '300px');
-            $selects.select2({width: 'resolve'});
+            $selects.not('.related-widget-wrapper select').select2({width: 'resolve'});
+            $selects.filter('.related-widget-wrapper.single select').select2({
+                // Keep updated labels after editing related obj
+                templateResult: function (data) {
+                    return $(data.element).text();
+                },
+                templateSelection: function (data) {
+                    return $(data.element).text();
+                },
+                width: 'resolve'
+            });
+            $selects.filter('.related-widget-wrapper.multiple select').select2({
+                templateResult: function (data) {
+                    return $(data.element).text();
+                },
+                templateSelection: function (data) {
+                    var $this = $(data.element).closest('.related-widget-wrapper');
+                    var siblings = $this.find('.change-related, .delete-related');
+                    if (!siblings.length) {
+                        return;
+                    }
+                    var value = data.id;
+                    var label = $(data.element).text();
+                    if (value) {
+                        siblings.each(function() {
+                            var elm = $(this);
+                            elm.attr('href', elm.attr('data-href-template').replace('__fk__', value));
+                            label += ' ';
+                            label += elm[0].outerHTML;
+                        });
+                    } else {
+                        siblings.removeAttr('href');
+                    }
+                    return label;
+                },
+                escapeMarkup: function(markup) {
+                    return markup;
+                },
+                width: '95%'
+            });
             $(el).find('select.select2').each(function(i, e) {
                 var opts = {};
                 if($(e).data('ajax-url')) {
@@ -184,6 +275,36 @@ var oscar = (function(o, $) {
                 if (!form || $(":invalid", form).length == 0)
                     $(this).button('loading');
             });
+
+            // Display tabs that have invalid input fields
+            $('input').on('invalid', function(){
+                var id = $(this).closest('.tab-pane').attr('id');
+                if (id) {
+                    $('.nav-list a[href="#' + id + '"]').tab('show');
+                }
+            });
+        },
+        initProductImages: function() {
+            // convert last 'extra' form into a multi-upload
+            // (assumes `extra=1` in django formset)
+            var $productImages = $('#product_images');
+            var $extraImg = $productImages.find('.upload-image li').last();
+            o.dashboard._extraProductImg = $extraImg.clone();
+
+            $productImages.find('a:disabled').parents('sortable-handle').sortable('disable');
+
+            $('ol.upload-image').sortable({
+                vertical: false,
+                group: 'serialization',
+                handle: '.btn-handle',
+                onDrop: function ($item, container, _super) {
+                    var $sortFields = $("input[name$=-display_order]");
+                    $sortFields.each(function(i){
+                        $(this).val(i);
+                    });
+                    _super($item, container);
+                }
+            });
         },
         offers: {
             init: function() {
@@ -321,25 +442,15 @@ var oscar = (function(o, $) {
             }
         },
         filereader: {
-            init: function() {
+            init: function () {
                 // Add local file loader to update image files on change in
                 // dashboard. This will provide a preview to the selected
                 // image without uploading it. Upload only occures when
                 // submitting the form.
                 if (window.FileReader) {
-                    $('input[type="file"]').change(function(evt) {
-                        var reader = new FileReader();
-                        var imgId = evt.target.id + "-image";
-                        reader.onload = (function() {
-                            return function(e) {
-                                var imgDiv = $("#"+imgId);
-                                imgDiv.children('img').attr('src', e.target.result);
-                            };
-                        })();
-                        reader.readAsDataURL(evt.target.files[0]);
-                    });
+                    $('input[type="file"]').change(onFileChange);
                 }
-            }
+            },
         },
         product_lists: {
             init: function() {
