@@ -2,15 +2,15 @@ from django.shortcuts import render
 
 
 def handler403(request, exception):
-    return render(request, '403.html', status=403)
+    return render(request, 'oscar/403.html', status=403)
 
 
 def handler404(request, exception):
-    return render(request, '404.html', status=404)
+    return render(request, 'oscar/404.html', status=404)
 
 
 def handler500(request):
-    return render(request, '500.html', status=500)
+    return render(request, 'oscar/500.html', status=500)
 
 
 def sort_queryset(queryset, request, allowed_sorts, default=None):
